/* 	LocalConnector.java
 * 
 * Created on: 22-nov-2006
 * Created by: Dennis Groenendijk
 * 
 * Copyright (c) 2002-2007, Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * 09-feb-2007	dennis@brains2b.nl      Load driver direct to for use with dynamic classloader
 * 26-02-2007   dennis@brains2b.nl      Username was returned lower case for oracle.
 * 07-04-2007   dennis@brains2b.nl      We were creating an empty entry in URL[]
 * 27-12-2007	dennis@brains2b.nl		07#11 Report error if driver is not available 		
*/
package org.brains2b.sql;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.sql.Connection;
import java.sql.Driver;
import java.sql.SQLException;
import java.util.Properties;
import java.util.StringTokenizer;

import org.brains2b.loader.DynamicClassLoader;
/**
 * Implementation of Connector to connect
 * to a local JDBC source, defined by the
 * parameters passed in the constructor
 * @author 	dennis@brains2b.nl
 * @version 1.0 [26-02-2007]
 */

public class LocalConnector implements Connector {

	private  Connection m_con;
    protected String m_url;
	protected Driver m_driver;
	protected Properties m_info;
    
    /**
     * Constructor which does not initialize
     * the connection properties.
     * <p>Only used for overwriting this class as needed.
     * Initialize the values yourself</p>
     * @see #init(String, String, String, String, String)
     */
    protected LocalConnector() {
        super();    
    }
    /**
     * Constructor
	 * <p>Requirement of the user and password parameters depends on 
     * the driver used. Set to null if not used.
     * @param driver String, the classname of the driver to use. 
     * <i>The class should be in the classpath</i>
     * @param url, the JDBC url describing which driver,server, port and database
     * to use. <i>See JDBC api description for syntax</i>
     * @param user, String the username to use for this connection
     * @param password, String the password to use for this connection     */
	public LocalConnector(String driver, String url, String user, String password) {
		super();
        init(null,driver,url,user,password);
	}
    
    /**
     * Constructor
     * <p>Requirement of the user and password parameters depends on 
     * the driver used. Set to null if not used.
     * @param classpath String, a semi-colon seperated list of jars and/or
     * dirs containing the driver
     * @param driver String, the classname of the driver to use. 
     * <i>The class should be in the supplied classpath or the default classpath</i>
     * @param url, the JDBC url describing which driver,server, port and database
     * to use. <i>See JDBC api description for syntax</i>
     * @param user, String the username to use for this connection
     * @param password, String the password to use for this connection
    */
    public LocalConnector(String classpath,String driver, String url, String user, String password) {
        super();
        init(classpath,driver,url,user,password);
    }
		

   /**
    * {@inheritDoc}
    */
	public Connection getConnection() throws SQLException {
		if (m_driver==null) {
			throw new SQLException("Cannot find driver");
		}
		if (m_con == null) {
			m_con =m_driver.connect(m_url, m_info);
			
		}
		return m_con;
	}


    /**
     * {@inheritDoc}
     */
	public void close() throws SQLException {
		if (m_con!=null) m_con.close();
		m_con =null;
	}
	
    /**
     * Initializes the parameters needed to make a connection.
     * <p>Requirement of the user and password parameters depends on 
     * the driver used.
     * @param classpath String, a semi-colon seperated list of jars and/or
     * dirs containing the driver
     * @param driver String, the classname of the driver to use. 
     * <i>The class should be in the classpath</i>
     * @param url, the JDBC url describing which driver,server, port and database
     * to use. <i>See JDBC api description for syntax</i>
     * @param user, String the username to use for this connection
     * @param password, String the password to use for this connection
     */
	protected void init(String classPath, String driver, String url, String user, String password) {
		m_url = url;
		m_info = new Properties();
        m_info.setProperty("user", user);
        m_info.setProperty("password", password);
		URL[] path = null;
		if (classPath!=null) {
			path = makeClassPath(classPath);
		}
       try {
            Class clazz=DynamicClassLoader.loadClass(driver,path);
            m_driver =(Driver) clazz.newInstance();
		} catch (ClassNotFoundException ce) {
			//XXX Errorhandling
			System.err.println(ce.getMessage());
		} catch (Exception ex) {
           ex.printStackTrace();
        }
	}

	/**
	 * Returns the name of the user this connection is using
	 * @return String, the name of the user
	 */
	public String getUser() {
        String s = m_info.getProperty("user");
        if (m_url.indexOf("oracle")!=-1) {
           return s.toUpperCase();
        }
        return s;
	}

    /**
     * get the database name this connection uses.
     * <p> Information is destilled from the url provided.
     * @return String, the name of the database
     * @since Squeler 0.23
     */
	public String getDbName() {
		if (m_url!=null) {
			int start=m_url.lastIndexOf('/')+1;
			if (start==0) {
				start = m_url.lastIndexOf(":")+1;
			}
			return m_url.substring(start);
		}
		return null;
		
	}
	
    /**
     * get the name of the server this connection uses.
     * <p> Information is destilled from the url provided.
     * @return String, the name of the server
     * @since Squeler 0.23
     */
	public String getServerName() {
		if (m_url!=null) {
			int start=m_url.lastIndexOf("@")+1;
			if (start==0) {
				start=m_url.indexOf("//")+2;
			}
			int end=m_url.indexOf(":",start);
			if (end==-1) {
				end=m_url.indexOf("/",start);
			} else {
				if (m_url.indexOf('.',start)!=-1) {
					end=m_url.indexOf('.',start);
				}
			}
			return m_url.substring(start,end);
		}
		return null;
	}

	/** Split up the classpath provided into seperate Strings
     * and make URLs from them  
     * @param classpath String, semi-colon seperated list of jars or dirs
     * containing the classes for this driver
     * @return URL[] the urls in this classpath
	 */
    private URL[] makeClassPath(String classpath) {
		try {
            StringTokenizer st= new StringTokenizer(classpath,";");
			URL[] urls=new URL[st.countTokens()];
			int cnt=0;
			while (st.hasMoreTokens()) {
				String s=st.nextToken();
				if (s.startsWith(System.getProperty("file.separator"))) {
					urls[cnt++]= new File(s).toURL();
				} else {
					urls[cnt++]= new File(System.getProperty("user.dir"),s).toURL();
				}
			}
			return urls;
		} catch (MalformedURLException mfuex) {
			return null;
		}
	}

    /**
     * {@inheritDoc}
     */
    public Object clone() {
        return new LocalConnector(m_driver.getClass().getName(),m_url,getUser(),m_info.getProperty("password"));
    }
}