/* 	SQLRecord.java
 * 
 * Created on: 12-02-2007
 * Created by: Dennis Groenendijk
 * 
 * Copyright (c) 2007, Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 /* @#History
 * [When]			[Who]				[What]
 * 23-10-2007		dennis@brains2b.nl	Implement hashCode
 * TODO Write JUnit test 
 * TODO String names for columns are threated as case sensitive
 */
package org.brains2b.sql;

import java.math.BigDecimal;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;



/** Object for containing results from a Select statement
 *  
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 1.01 [23-10-2007]
 */
public class SQLRecord implements Cloneable{
	
    private Object[] m_record;

    /**
     * Constructor for SQLRecord.
     * <p>Sets the initial size to 0</p>
     */
    public SQLRecord() {
        this(0);
    }
    
    /**
     * Constructor for SQLRecord.
     * @param initialSize int, the number of columns to
     * create for this Record
     */
    public SQLRecord(int initialSize) {
        super();
        m_record=new Object[initialSize];
    }

	/**
	 * get the number of fields in this cursor data object
	 * @return int, the number of columns
	 */
	public int getColumnCount() {
		return m_record.length/3;
	}
    
    /**
     * set a column identified by name at the specified position
     * @param idx int, the index to set the column to
     * @param name String, the identifier for this column
     */
    public void setColumn(int idx, String name) {
    	if (idx>=m_record.length/3) {
    		resize(idx+1);
    	}
    	m_record[idx*3]=name;
    }
    
    /**
     * get the column identifier for the column at the given index
     * @param idx int, the index of the column
     * @return String, the column identifier
     */
    public String getColumn(int idx) {
    	return (String) m_record[idx*3];
    }
    
    /** 
     * set the Class identifying the type of content for this column
     * by index.
     * @param idx int, the index of the column
     * @param c Class, the class identifying the content for this column.
     * Use the Integer.TYPE, Boolean.TYPE, etc. for the primitives
     */
     public void setColumnType(int idx, Class c) {
    	if (idx>=m_record.length/3) {
            resize(idx+1);
        }
    	m_record[idx*3+1]=c;
    }
    
    /**
     * get the column type for the column identified by name
     * @param name String, the column identifier
     * @return Class, the Class-definition of the column, for primitives are returned
     * as Integer.TYPE, Double.TYPE, etc.
     */
    public Class getColumnType(String name) {
    	int idx=getIndex(name);
    	return (Class) m_record[idx*3+1];
    }
	
	/**
     * get the column type for the column given by the specified index
     * @param idx int, the index of the column
     * @return Class, the Class-definition of the column, for primitives are returned
     * as Integer.TYPE, Double.TYPE, etc.
     */
	public Class getColumnType(int idx) {
		return (Class) m_record[idx*3+1];
    }
   
   	/**
   	 * set the content value for the specified index
   	 * @param idx int, the index of the column
   	 * @param value Object, the value for this specific column. The type of Object
   	 * should be consistent with the type of column you defined through <code>setColumnType(int,Class)</code>
   	 * @see #setColumnType(int, Class)
   	 */
   	public void setValue(int idx, Object value) {
   		if (idx>=m_record.length/3) {
            resize(idx+1);
        }
   		m_record[idx*3+2]=value;
   	}

	/**
   	 * set the content value for the column identified by name
   	 * @param name String, the column identifier
   	 * @param value Object, the value for this specific column. The type of Object
   	 * should be consistent with the type of column you defined through <code>setColumnType(int,Class)</code>
   	 * @see #setColumnType(int, Class)
   	 */
	public void setValue(String name, Object value) {
		setValue(getIndex(name),value);	
	}   
	
	/** convinience method to set the column name,class and value at once at a given index
	 * @param idx int, the index of the column
	 * @param name String, the column identifier
	 * @param value Object, the value for this specific column. You can set the column type
	 * afterwards. The value and type should be consistent
	 */
	public void setColumn(int idx, String name, Class c, Object value) {
		setColumn     ( idx, name );
        setColumnType ( idx, c);
		setValue      ( idx, value);
	} 
    
    /** convinience method to set the column name,class and value at once at a given index
     * @param idx int, the index of the column
     * @param name String, the column identifier
     * @param value Object, the value for this specific column. You can set the column type
     * afterwards. The value and type should be consistent
     */
    public void setColumn(int idx, String name, Class c) {
        setColumn     ( idx, name );
        setColumnType ( idx, c);
    } 

	/**
	 * get the value of the column at the given index as an Object
	 * @param idx int, the index of the column
	 * @return Object, the value of this column, will return <code>null</code> if no
	 * previous value was set 
	 */
	public Object getValue(int idx) {
	    return m_record[idx*3+2];
	}
	
     /**
     * get the boolean value belonging to the column identified by name
     * @param name String, the name of the column
     * @return boolean, the value for this column, false if the column
     * is not defined (yet)
     * @throws NumberFormatException, If the value set to this specific column cannot be cast to an <code>int</code>
     */
    public boolean getBoolValue(String name) {
        final int idx= getIndex(name);
        if (idx==-1) return false;
        return getBoolValue(idx);
    }
    
    /**
     * get the value of the column at the given index as a boolean.
     * @param idx int, the index of the column
     * @return double, the boolean value for this column, false if the column
     * is not defined (yet)
     * @throws NumberFormatException, If the value set to this specific column cannot be cast to an <code>int</code>
     */
    public boolean getBoolValue(int idx) {
        final Object value= getValue(idx);
        if (value==null) return false;
        if (value instanceof Boolean) {
            return ((Boolean) value).booleanValue();
        }else if (value instanceof String) {
            return Boolean.valueOf((String) value).booleanValue();
        } else {
            throw new NumberFormatException("Value is not a boolean");
        }
    }
    
    /**
     * get the value of the column at the given index as a double.
     * @param idx int, the index of the column
     * @return double, the double value for this column, 0 (zero) if the column
     * is not defined (yet)
     * @throws NumberFormatException, If the value set to this specific column cannot be cast to an <code>int</code>
     */
	public double getDoubleValue(int idx) {
	    final Object value= getValue(idx);
        if (value==null) return 0d;
		if (value instanceof BigDecimal) {
			return ((BigDecimal) value).doubleValue();
		} else {
			throw new NumberFormatException("Value is not a number");
		}
	}
	
     /**
     * get the double value belonging to the column identified by name
     * @param name String, the name of the column
     * @return double, the value for this column, 0 (zero) if the column
     * is not defined (yet)
     * @throws NumberFormatException, If the value set to this specific column cannot be cast to an <code>int</code>
     */
	public double getDoubleValue(String name) {
		final int idx= getIndex(name);
		if (idx==-1) return 0d;
        return getDoubleValue(idx);
	}
	
	/**
	 * get the value of the column at the given index as an integer.
	  @param idx int, the index of the column
	 * @return int, the int value for this column, 0 (zero) if the column
     * is not defined (yet)
	 * @throws NumberFormatException, If the value set to this specific column cannot be cast to an <code>int</code>
	 */
	public int getIntValue(int idx) {
		final Object value= getValue(idx);
        if (value==null) return 0;
        if (value instanceof BigDecimal) {
            return ((BigDecimal) value).intValue();
        } else if (value instanceof Integer) {
            return ((Integer) value).intValue();
        } else if (value instanceof Short) {
            return ((Short) value).intValue();
        } else {
            throw new NumberFormatException("Value is not a number");
        }
	}
	
	/**
	 * get  the value of the column at the given index as a long.
	 * @param idx int, the index of the column
	 * @return long, the long value for this column, 0 (zero) if the column
     * is not defined (yet)
	 * @throws NumberFormatException, If the value set to this specific column cannot be cast to an <code>int</code>
	 */
	public long getLongValue(int idx) {
	    final Object value= getValue(idx);
        if (value==null) return 0L;
        if (value instanceof BigDecimal) {
            return ((BigDecimal) value).longValue();
       
        } else {
            throw new NumberFormatException("Value is not a number");
        }
	}
	
	 /**
     * get the longValue belonging to the column identified by name
     * @param name String, the name of the column
     * @return long, the value for this column, 0 (zero) if the column
     * is not defined (yet)
     * @throws NumberFormatException, If the value set to this specific column cannot be cast to an <code>int</code>
     */
	public long getLongValue(String name) {
		final int idx= getIndex(name);
        if (idx==-1) return 0L;
		return getLongValue(idx);
	}
    
    
    public String getString(String name) {
        return (String) getValue(name);
    }
    
    public String getString(int idx) {
        return (String) getValue(idx);
    }
	
	/**
	 * get the value of the column identified by name as an Object
	 * @param name String, the column identifier
	 * @return Object, the value for this column
	 */
	public Object getValue(String name) {
		final int idx= getIndex(name);
        if (idx==-1) return null;
		return m_record[idx*3+2];
	}

	
    /**
     * get the intValue belonging to the column identified by name
     * @param name String, the name of the column
     * @return int, the value for this column, 0 (zero) if the column
     * is not defined (yet)
     * @throws NumberFormatException, If the value set to this specific column cannot be cast to an <code>int</code>
     */
	public int getIntValue(String name) {
		final int idx= getIndex(name);
        if (idx==-1) return 0;
		return getIntValue(idx);
	}

	/** get the index of the column identified by name
	 * @param name String, the column identifier ignoring case
	 * @return int, the index of the column
	 */
	public int getIndex(String name) {
        if (name==null) return -1;
		for (int i=0;i<m_record.length;i+=3) {
			if (name.equalsIgnoreCase((String)m_record[i])) {
				return i/3;
			}
		}
	   return -1;	
    }
	

	/**
	 * determines if two objects are equal by comparing their hashCodes
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(Object obj) {
        if (hashCode()==obj.hashCode()) {
        	return true;
        } 
        return false;
    }
    
    public int hashCode() {
    	int hashCode = 13;
    	for (int i=0;i<m_record.length;i++) {
    		if (m_record[i]!=null)
    		hashCode += i*43+m_record[i].hashCode();
    	}
    	return hashCode;
    }

    /**
     * grow the array which stores the content
     * @param idx int, the new total size
     */
    private final void resize(int idx) {
        if (idx<=m_record.length/3) return;
        Object[] record=new Object[idx*3];
        System.arraycopy(m_record, 0, record, 0, m_record.length);
        m_record=record;
    }
    
    /**
     * clone the definition of this record, but not
     * the values set for any of the columns
     * @return SQLRecord
     */
    public SQLRecord cloneDefinition() {
        SQLRecord record= new SQLRecord(getColumnCount());
        for (int i=0;i<getColumnCount();i++) {
            record.setColumn(i, getColumn(i), getColumnType(i));
        }
        return record;
    }
    
    /**
     * {@inheritDoc}
     */
    public Object clone() {
        SQLRecord record= new SQLRecord(getColumnCount());
        for (int i=0;i<getColumnCount();i++) {
            record.setColumn(i, getColumn(i), getColumnType(i), getValue(i));
        }
        return record;
    }
    
    /**
     * automatically create a new SQLRecord from a ResultSet
     * @param rs ResultSet, expects a previous successful call to rs.next()
     * @return SQLRecord, a record filled with the same values as the resultset
     * @throws SQLException thrown if ResultSet throws the Exception
     */
    public static final SQLRecord createRecord(ResultSet rs) throws SQLException {
        try {
            ResultSetMetaData meta = rs.getMetaData();
            SQLRecord record= new SQLRecord(meta.getColumnCount());
            for (int i=1;i<meta.getColumnCount()+1;i++) {
                record.setColumn(i-1, 
                        meta.getColumnName(i),
                        Class.forName(meta.getColumnClassName(i)),
                        rs.getObject(i));
            }
            return record;
        } catch (ClassNotFoundException cnfex) {
            throw new SQLException("Cannot determine datatype for one of the fields");
        }
    }
    
    /**
     * automatically create a new SQLRecord structure from ResultSetMeta
     * <p>The name and type are set, the value for this column is not set.
     * This can be used when the resultset is empty and a new record needs
     * to be created.
     * @param rs ResultSetMeta
     * @return SQLRecord, a record filled with the same values as the resultset
     * @throws SQLException thrown if ResultSet throws the Exception
     */
    public static final SQLRecord createEmptyRecord(ResultSetMetaData meta) throws SQLException {
        try {
            SQLRecord record= new SQLRecord(meta.getColumnCount());
            for (int i=1;i<meta.getColumnCount()+1;i++) {
                record.setColumn(i-1, 
                        meta.getColumnName(i),
                        Class.forName(meta.getColumnClassName(i)));
            }
            return record;
        } catch (ClassNotFoundException cnfex) {
            throw new SQLException("Cannot determine datatype for one of the fields");
        }
    }
    
}
