/* 	ColumnModel.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 25-mrt-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * May 5, 2007	dennis@brains2b.nl      Changes from test
 * May 14,2007  dennis@brains2b.nl      decode default in getter
 * Jun 06,2007  dennis@brains2b.nl      simple toString() implementation
 * 17-10-2007	dennis@brains2b.nl		correct datatype supplied as OTHER for LOB's as needed
 * 26-10-2007	dennis@brains2b.nl		Implemented equals()	
*/
package org.brains2b.sql.meta;

import java.sql.Types;

import org.brains2b.sql.SQLHelper;
import org.brains2b.sql.SQLRecord;

/** Wrapper around column definition from DatabaseMetaData
 * <p>Class is read-only, use SquelerDDL:ColumnDDL for changing columns</p>
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 1.01 [26-10-2007]
 * @see SQLMetaData
 * @see SQLRecord
 */
public class ColumnModel implements SQLMetaModel {

    private SQLRecord m_record;
    
    /**
     * Constructor 
     * only used by ColumnDDL
     *
     */
    protected ColumnModel() {
        super();
    }
    
    /**
     * Create a ColumnModel on the basis of this SQLRecord
     * @param record SQLRecord, the record containing the 
     * information on this column
     */
    public ColumnModel(SQLRecord record) {
        super();
        m_record = record;
        if (getDataType()==Types.OTHER && getTypeName()!=null) {
        	correctOther(getTypeName());
        }
    }
    
    /**
     * {@inheritDoc}
     */
    public String getName() {
        return m_record.getString("COLUMN_NAME");
    }

    /**
     * {@inheritDoc}
     */
    public SQLRecord getRecord() {
        return m_record;
    }

    /**
     * {@inheritDoc}
     */
    public String getType() {
        return "COLUMN";
    }

    /**
     * get the sequence of this column within the object it is
     * contained in such as a {@link TableModel}, {@link IndexModel} or
     * {@link ConstraintModel}
     * @return int, a number starting at 1 (not 0) of the position of
     * this column as part of another object
     */
    public int getSequence() {
        return m_record.getIntValue("ORDINAL_POSITION");
    }

    /**
     * get the table name
     * @return String, the name of the table
     */
    public String getTableName() {
        return (String) getRecord().getValue("TABLE_NAME");
    }

    /**
     * get the datatype as defined in <code>java.sql.Type</code>
     * @return int, the type of the data in this column
     */
    public int getDataType() {
        return getRecord().getIntValue("DATA_TYPE");
    }

    /**
     * get the type description for the column type
     * <p>This value is specific for the database used
     * @return String, the type description
     */
    public String getTypeName() {
        return (String) getRecord().getValue("TYPE_NAME");
    }

    /**
     * get the size of the column for non-numeric types or the precision for numeric types
     * @return int, the size or precision for this column
     */
    public int getColumnSize() {
        return getRecord().getIntValue("COLUMN_SIZE");
    }

    /**
     * get the number of decimal places for a numeric field.
     * @return int, the number of decimal places
     */
    public int getDecimalDigits() {
        return getRecord().getIntValue("DECIMAL_DIGITS");
    }

    /**
     * get an indication if the field is nullable
     * @return String, indicator if the column can be nullable
     */
    public boolean isNullable() {
        return getRecord().getString("IS_NULLABLE").equals("YES");
    }

    /**
     * get optional remarks for a column
     * @return String, the remarks for this column
     */
    public String getRemarks() {
        return (String) getRecord().getValue("REMARKS");
    }

    /**
     * get the column default value
     * @return String, a description of the column definition
     */
    public String getDefault() {
        return SQLHelper.stripQuotes((String) getRecord().getValue("COLUMN_DEF"));
    }
    
    /**
     * a String representation of this column as <code>tablename.name</code>
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new StringBuffer().append(getTableName()).append('.').append(getName()).toString();
    }

    /** set the record for this Model
     *  @param record SQLRecord
     */
    protected void setRecord(SQLRecord record) {
        m_record = record;
    }
    private final void correctOther(String typeName) {
    	if ("BLOB".equals(typeName)) {
    		m_record.setValue("DATA_TYPE", new Integer(Types.BLOB));
    	} else if ("CLOB".equals(typeName)) {
    		m_record.setValue("DATA_TYPE", new Integer(Types.CLOB));
    	}
    }
    
    public boolean equals(Object obj) {
    	if (!(obj instanceof ColumnModel)) return false;
    	ColumnModel tm = (ColumnModel) obj;
    	return m_record.equals(tm.getRecord());
    }
}
