/* 	ConstraintModel.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 25-mrt-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]       [Who]                   [What]
 * May 5, 2007  dennis@brains2b.nl      Changes from test  
 * 26-10-2007	dennis@brains2b.nl		Implemented equals()     
*/
package org.brains2b.sql.meta;

import java.util.ArrayList;
import java.util.List;

import org.brains2b.sql.SQLRecord;

/** Wrapper around the constraint definition from DatabaseMetaData
 * <p>Class is read-only, use SquelerDDL:ConstraintDDL for changing columns</p>
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 1.01 [26-10-2007]
 * @see SQLMetaData
 * @see SQLRecord
 */
public class ConstraintModel implements SQLMetaModel {
    
    /** constant for a primary key constraint */
    public final static String PRIMARY_KEY = "PRIMARY KEY";
    /** constant for a foreign key constraint */
    public final static String FOREIGN_KEY = "FOREIGN KEY";
    /** constant for a check constraint */
    public final static String CHECK       = "CHECK";
    
    private SQLRecord m_record;
    private List m_columns;
    
    /**
     * Constructor 
     * only used by ConstraintDDL
     *
     */
    protected ConstraintModel() {
        super();
    }

    /**
     * Create a ConstriantModel of this specific type
     * on the basis of the provided SQLRecord
     * @param type String, one of the constants (PRIMARY_KEY|FOREIGN_KEY|CHECK)
     * @param record SQLRecord, the record describing this constraint
     * @throws IllegalArgumentException if type is not in (PRIMARY_KEY|FOREIGN_KEY|CHECK)
     */
    public ConstraintModel(String type, SQLRecord record) {
        super();
        if (!PRIMARY_KEY.equals(type) && !FOREIGN_KEY.equals(type) && !CHECK.equals(type)) {
            throw new IllegalArgumentException("Constraint type must be " + PRIMARY_KEY +", " + FOREIGN_KEY +" or " + CHECK);
        }
        m_record=record;
        m_record.setColumn(m_record.getColumnCount(), "TYPE", String.class, type);
    }
    
    /**
     * {@inheritDoc}
     */
    public String getName() {
        return (String) m_record.getValue("NAME");
    }

    /**
     * {@inheritDoc}
     */
    public SQLRecord getRecord() {
        return m_record;
    }

    /**
     * get the table name for this Constraint
     * @return String
     */
    public String getTableName() {
        return m_record.getString("TABLE_NAME");
    }
    
    /**
     * {@inheritDoc}
     */
    public String getType() {
        return m_record.getString("TYPE");
    }
    
    /**
     * get the Columns referenced by this constraint
     * @return List of ColumnModel, null if this is 
     * constraint is of type <code>CHECK</code>
     */
    public List getColumns() {
        return m_columns;
    }

    /** set the record for this Model
     *  @param record SQLRecord
     */
    protected void setRecord(SQLRecord record) {
        m_record = record;
    }
    
    /**
     * set the columns for this Model
     * @param columns List
     */
    protected void setColumns(List columns) {
        m_columns=columns;
    }
    
    /**
     * add a ColumnModel as a part of this ConstraintModel
     * @param model ColumnModel
     */
    protected void addColumn(ColumnModel model) {
        if (m_columns==null) {
            m_columns = new ArrayList();
        }
        m_columns.add(model);
    }
    
    public boolean equals(Object obj) {
    	if (!(obj instanceof ConstraintModel)) return false;
    	ConstraintModel tm = (ConstraintModel) obj;
    	boolean b = m_record.equals(tm.getRecord());
    	if (!b || m_columns==null || m_columns.isEmpty()) return b;
    	List cols = tm.getColumns();
    	if (cols==null || cols.size()!=m_columns.size()) return false;
    	for (int i=0; i<m_columns.size();i++) {
    		b = m_columns.get(i).equals(cols.get(i));
    		if (!b) return b;
    	}
    	return true;
    }
}
