/* 	IndexModel.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 27-mrt-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]       [Who]                   [What]
 * May 5, 2007  dennis@brains2b.nl      Changes from test
 * 26-10-2007	dennis@brains2b.nl		Implemented equals()    
*/
package org.brains2b.sql.meta;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.brains2b.sql.SQLRecord;

/** Wrapper around the index definition from DatabaseMetaData
 * <p>Class is read-only, use SquelerDDL:IndexDDL for changing columns</p>
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 1.01 [26-10-2007]
 * @see SQLMetaData
 * @see SQLRecord
 */
public class IndexModel implements SQLMetaModel {

    private SQLRecord m_record;
    private List m_columns;
    
     /**
     * Constructor 
     * only used by IndexDDL
     *
     */
    protected IndexModel() {
        super();
    }
    
    /**
     * Create an IndexModel on the basis of this SQLRecord
     * @param record SQLRecord, the record containing the 
     * information on this column
     */
    public IndexModel(SQLRecord record) {
       super();
       m_record = record;
    }
    
    /**
     * {@inheritDoc}
     */
    public String getName() {
        return m_record.getString("INDEX_NAME");
    }

    /**
     * {@inheritDoc}
     */
    public SQLRecord getRecord() {
        return m_record;
    }

    /**
     * {@inheritDoc}
     */
    public String getType() {
        return "INDEX";
    }

    /**
     * get the columns involved in this Index
     * @return List of ColumnModel
     */
    public List getColumns() {
        return m_columns;
    }
    
    /**
     * set the columns for this Model
     * @param columns List
     */
    protected void setColumns(List columns) {
        m_columns=columns;
    }

    /**
     * get the table for this index
     * @return String
     */
    public String getTableName() {
        return m_record.getString("TABLE_NAME");
    }

    /**
     * check if this is an unique index
     * @return boolean
     */
    public boolean isUnique() {
        if (m_record.getValue("NON_UNIQUE") instanceof Integer
                ||m_record.getValue("NON_UNIQUE") instanceof BigDecimal
                ||m_record.getValue("NON_UNIQUE") instanceof Short) {
            return m_record.getIntValue("NON_UNIQUE")==0?true:false;
        } 
        return m_record.getBoolValue("NON_UNIQUE");
    }

    /**
     * add a ColumnModel as a part of this IndexModel
     * @param model ColumnModel
     */
    protected void addColumn(ColumnModel model) {
        m_columns.add(model);   
    }

    /**
     * set a column at a specific position for this index
     * @param position, a numeric position for this column starting at 0
     * @param columnName String , the name of the column to add
     */
    void setColumn(int position, ColumnModel model) {
    	if (m_columns ==null) {
    		m_columns=new ArrayList();
    	}
        m_columns.add(position-1,model);    
    }

    /** set the record for this Model
     *  @param record SQLRecord
     */
    protected void setRecord(SQLRecord record) {
        m_record = record;
    }
    
    public boolean equals(Object obj) {
    	if (!(obj instanceof IndexModel)) return false;
    	IndexModel tm = (IndexModel) obj;
    	boolean b = m_record.equals(tm.getRecord());
    	if (!b || m_columns==null || m_columns.isEmpty()) return b;
    	List cols = tm.getColumns();
    	if (cols==null || cols.size()!=m_columns.size()) return false;
    	for (int i=0; i<m_columns.size();i++) {
    		b = m_columns.get(i).equals(cols.get(i));
    		if (!b) return b;
    	}
    	return true;
    }
}
