/* 	ProcedureModel.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 31-mrt-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * 26-10-2007	dennis@brains2b.nl		Implemented equals()
 * TODO write JUnit test		
*/
package org.brains2b.sql.meta;

import java.sql.DatabaseMetaData;
import java.util.List;

import org.brains2b.sql.SQLRecord;

/** Wrapper around a procedure definition from DatabaseMetaData
 * <p>Class is read-only, use SquelerDDL:ProcedureDDL for changing columns</p>
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 1.01 [26-10-2007]
 * @see SQLMetaData
 * @see SQLRecord
 */
public class ProcedureModel implements SQLMetaModel {

    /** constant for a model defined as procedure */
    public final static String PROCEDURE="PROCEDURE";
    /** constant for a model defined as function */
    public final static String FUNCTION="FUNCTION";
    /** constant for a model defined as package */
    public final static String PACKAGE="PACKAGE";
    /** constant for a model defined as package body */
    public final static String PACKAGE_BODY="PACKAGE BODY";
    
    private SQLRecord m_record;
    private List parameters;
    private ParameterModel returnValue;
    
    /**
     * Constructor 
     * only used by ProcedureDDL
     *
     */
    protected ProcedureModel() {
        super();
    }
    
    /**
     * Create a ProcedureModel on the basis of this SQLRecord
     * <p>If the PROCEDURE_TYPE of this SQLRecord equals 
     * {@link DatabaseMetaData#procedureReturnsResult} type
     * is set to {@link #FUNCTION}</p> 
     * @param record SQLRecord, the record containing the 
     * information on this column
     */
    public ProcedureModel(SQLRecord record) {
        super();
        m_record = record;
        
        String type = PROCEDURE;
        if (m_record.getIntValue("PROCEDURE_TYPE")==DatabaseMetaData.procedureReturnsResult) {
            type = FUNCTION;
        }
        m_record.setColumn(m_record.getColumnCount(), "TYPE", String.class, type);
    }
    
    /**
     * {@inheritDoc}
     */
    public String getName() {
        return m_record.getString("PROCEDURE_NAME");
    }

    /**
     * {@inheritDoc}
     */
    public SQLRecord getRecord() {
        return m_record;
    }

    /**
     * {@inheritDoc}
     */
    public String getType() {
        return m_record.getString("TYPE");
    }

    /**
     * get the parameters for this ProcedureModel
     * @return List of ParameterModel
     */
    public List getParameters() {
    	return parameters;
    }

    /**
     * set the parameters for this ProcedureModel
     * @param list List of ParameterModel
     */
    protected void setParameters(List list) {
    	parameters = list;
    }

    /**
     * get the return value for this ProcedureModel
     * @return ParameterModel, null if this ProcedureModel
     * is not a function
     */
    public ParameterModel getReturnValue() {
    	return returnValue;
    }

    /**
     * set the return value for this ProcedureModel
     * @param model ParameterModel
     */
    protected void setReturnValue(ParameterModel model) {
    	returnValue = model;
    }

    /**
     * get the Catalog for this ProcedureModel
     * @return String, null if this procedure does not
     * have a catalog 
     */
    public String getCatalog() {
        return m_record.getString("PROCEDURE_CAT");
    }

    /**
     * get the Schema for this ProcedureModel
     * @return String, null if this procedure does not
     * have a schema 
     */
    public String getSchema() {
        return m_record.getString("PROCEDURE_SCHEM");
    }
    
    public boolean equals(Object obj) {
    	if (!(obj instanceof ParameterModel)) return false;
    	ProcedureModel tm = (ProcedureModel) obj;
    	boolean b = m_record.equals(tm.getRecord());
    	if (!b || parameters==null || parameters.isEmpty()) return b;
    	List cols = tm.getParameters();
    	if (cols==null || cols.size()!=parameters.size()) return false;
    	for (int i=0; i<parameters.size();i++) {
    		b = parameters.get(i).equals(cols.get(i));
    		if (!b) return b;
    	}
    	return true;
    }

}
