/* 	SqlMetaData.java
 *	Copyright (c) 2001-2007 Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]				[What]
 * 11/03/2002	dennis@brains2b.nl	getColumns did not use getMetaData
 * 														closed the cursor.
 * 31-01-2003	dennis@brains2b.nl	do not generate ROWID for views
 * 24-03-2004	dennis@brains2b.nl	close all resultsets after execution
 * 27-10-2004	dennis@brains2b.nl	close resultset after setParameters
 * 18-07-2005	dennis@brains2b.nl	Fix for apparent bug in Oracle driver.
 * 									Leaves tableType cursor open. cache it here
 * 30-08-2006	dennis@brains2b.nl	Only set datatypes when not set previously
 * 30-01-2007   dennis@brains2b.nl  #909 Implemented using default values
 * 30-04-2007   dennis@brains2b.nl  Correct creating ROWID for new ColumnModel
 * May 5, 2007  dennis@brains2b.nl  Changes from test
 * Jun 11, 2007  dennis@brains2b.nl  Fix for problem with Oracle indexes,
 *                                  Don't automaticly load parameters with procedure   
 * 22-06-2007   dennis@brains2b.nl  Use getImportedKeys instead of getExportedKeys
 * 16-07-2007   dennis@brains2b.nl  Fix problem with pseudokeys recognition and adding primary keys
 * 17-01-2008	dennis@brains2b.nl  changes for new TypesInfo                                  
 */
package org.brains2b.sql.meta;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.brains2b.log.Logger;
import org.brains2b.sql.SQLRecord;

/**
 * Main DatabaseMetaData class for describing database, table 
 * and column properties on the basis of a supplied connection
 * 
 * @author  Dennis Groenendijk
 * @since   22-06-2007
 * @version 0.22.4
 *
 */
public class SQLMetaData {
    
    private DatabaseMetaData m_dbMeta;
    private Connection m_con;
    private String[] m_types;
    private PreparedStatement m_prep;
    private TypesInfo m_sqlTypes;
    /**
     * Default Constructor
     * <p>Use {@link #setConnection(Connection)} prior
     * to using any of the methods</p> 
     */
    public SQLMetaData() {
        super();
    }
    
    /**
     * Constructor for SqlMetadata 
     * @param connection Connection, the database connection used
     */
	public SQLMetaData(Connection connection) {
		super();
        m_con=connection;
		m_dbMeta=null;
		
	}
    
    /**
     * get the DatabaseMetaData object associated with the connection used
     * @return DatabaseMetaData
     * @throws SQLException, from the connection
     */
    public DatabaseMetaData getMetaData() throws SQLException {
        if (m_dbMeta==null) {
            m_dbMeta=m_con.getMetaData();
        }
        return m_dbMeta;
    }
    
    /**
     * get the catalog names which can be retrieved for this database
     * @return List of Strings, always returns a List even when empty
     * @throws SQLException
     */
    public List getCatalogs() throws SQLException {
        ArrayList list=new ArrayList();
        ResultSet rs= getMetaData().getCatalogs();
        while (rs.next()) {
            list.add(rs.getString("TABLE_CAT"));
        }
        rs.close();
        return list;
    }
    /**
     * get the schema names which can be retrieved for this database
     * @return List of Strings, always returns a List even when empty
     * @throws SQLException
     */
    public List getSchemas() throws SQLException {
        ArrayList list = new ArrayList();
        ResultSet rs= getMetaData().getSchemas();
        while (rs.next()) {
            list.add(rs.getString("TABLE_SCHEM"));
        }
        rs.close();
        return list;
    }
    /**
     * get the table types supported by the database
     * <p><i>Note:</i>Types does not return synonym</p>
     * @return List of Strings, always returns a List even when empty
     */
    public List getTableTypes() throws SQLException {
        ArrayList list=new ArrayList();
        ResultSet rs= getMetaData().getTableTypes();
        while (rs.next()) {
            list.add(rs.getString("TABLE_TYPE"));
        }
        // Remove synonym if found, information cannot be used to get Column data (Oracle only?)
        list.remove("SYNONYM");
        rs.close();
        return list;
    }
    
    /**
     * get the table types supported by the database as an array
     * <p><i>Note:</i>Types does not return synonym</p>
     * @return String[], always returns an array even when empty
     */
    private String[] getTypes() throws SQLException {
    	if (m_types==null) {
    		List list=getTableTypes();
    		m_types=new String[list.size()];
    		list.toArray(m_types);
    	}
    	return m_types;	
    }
    
    /**
     * get a List of TableModel objects, including their columns
     * dependant on the parameters supplied. 
     * <p>Setting any of the parameters to null will ignore them.
     * Setting either catalog or schemaPattern is usually necessary.
     * @param catalog String, a possible catalog name, null if not used
     * @param schemaPattern String, a possible schemaPattern, null if not used
     * @param tablePattern, a possible tablePattern, null if not used.
     * @return List of TableModels, always returns a List even when empty
     * @throws SQLException
     * @see #getTables, #getColumns
     * @see java.sql.DatabaseMetaData for information on catalogs and patterns
     */
    public List getTablesWithColumns(String catalog, String schemaPattern,
    	String tablePattern) throws SQLException {
    	List list=getTables(catalog,schemaPattern,tablePattern);
    	Iterator it=list.iterator();
		while (it.hasNext()) {
			TableModel model=(TableModel) it.next();
			model.setColumns(getColumns(model.getCatalog(),
				model.getSchema(),
				model.getName(),
				null));
		}
		return list;
		
   	}
    
    /**
     * get a List of TableModel objects, without their columns
     * dependant on the parameters supplied. 
     * <p>Setting any of the parameters to null will ignore them.
     * Setting either catalog or schemaPattern is usually necessary.
     * @param catalog String, a possible catalog name, null if not used
     * @param schemaPattern String, a possible schemaPattern, null if not used
     * @param tablePattern, a possible tablePattern, null if not used.
     * @return List, a List of SqlTableModels, always returns a List even when empty
     * @throws SQLException
     * @see java.sql.DatabaseMetaData for information on catalogs and patterns
     */
    public List getTables(String catalog, String schemaPattern,
        String tablePattern) throws SQLException {
        return getTables(catalog,schemaPattern,tablePattern,null);
    }
    
    /**
     * get a List of TableModel objects for a specific type, without their columns
     * dependant on the parameters supplied. 
     * <p>Setting any of the parameters to null will ignore them.
     * Setting either catalog or schemaPattern is usually necessary.
     * @param catalog String, a possible catalog name, null if not used
     * @param schemaPattern String, a possible schemaPattern, null if not used
     * @param tablePattern, a possible tablePattern, null if not used.
     * @return List, a List of SqlTableModels, always returns a List even when empty
     * @throws SQLException
     * @see java.sql.DatabaseMetaData for information on catalogs and patterns
     * @see {@link #getTableTypes()}
     */
    public List getTables(String catalog, String schemaPattern,
        String tablePattern, String type) throws SQLException {
    
    	String[] types=null;
    	if (type==null) {
        types=getTypes();
    	} else {
    		types=new String[]{type};
    	}
        ArrayList list=new ArrayList();
        ResultSet rs=getMetaData().getTables(catalog,schemaPattern,tablePattern,types);
        while(rs.next()) {
            
            TableModel tm =new TableModel(SQLRecord.createRecord(rs));
            list.add(tm);
        }
        rs.close();
        return list;
    }
    
    /**
     * set the columns for this table
     * @param proc ProcedureModel, the model to set the parameters to
     * @throws SQLException
     * @see {@link #getColumns(String, String, String, String)}
     */    
    public void setColumns(TableModel table) throws SQLException{
        List columns = getColumns(table.getCatalog(), table.getSchema(), table.getName(), null);
        table.setColumns(columns);
    }
    
    /**
     * get a List of SqlColumModels for each colum that can be found for the parameters supplied. 
     * <p>Setting any of the parameters to null will ignore them.
     * Setting either catalog or schemaPattern is usually necessary.
     * @param catalog String, a possible catalog name, null if not used
     * @param schemaPattern String, a possible schemaPattern, null if not used
     * @param tablePattern, a possible tablePattern, null if not used.
     * @param columnPattern, a possible columnPattern, null if not used.
     * @return Vector, a List of SqlTableModels, always returns a Vector even when empty
     * @throws SQLException
     * @see java.sql.DatabaseMetaData for information on catalogs and patterns
     */
    public List getColumns(String catalog, String schemaPattern,
        String tablePattern, String columnPattern) throws SQLException {
        ArrayList list=new ArrayList();
        ResultSet rs=getMetaData().getColumns(catalog,schemaPattern,tablePattern
        ,columnPattern);
        while(rs.next()) {
            ColumnModel column = new ColumnModel(SQLRecord.createRecord(rs));
            if (column.getDataType()==Types.DATE && isOracleDriver()) {
            	column.getRecord().setValue("DATA_TYPE", new Integer(Types.TIMESTAMP));
            }
            list.add(column);
        }
        rs.close();
        return list;
    }
    
    /**
     * set the primary key belonging to a table
     * <p> The method returns the table model enriched with a possible primary key. 
     * If no primary key can be found a primary key is constructed from possiblem unique indexes. 
     * If no unique indexes where defined a pseudo column called ROWID is created.
     * @param tm TableModel, the table to find the primary key for
     * @return TableModel with the primary key set
     * @throws SQLException
     */
    public void setPrimaryKey(TableModel tm)  throws SQLException {
    	setPrimaryKey(tm,false);
    }
    /**
     * set the primary key belonging to a table
     * <p> The method enriches the TableModel with a possible primary key. 
     * If no primary key can be found a primary key is constructed from possiblem unique indexes. 
     * If no unique indexes where defined a pseudo column called ROWID is created.
     * @param tm TableModel, the table to find the primary key for
     * @throws SQLException
     */
    public void setPrimaryKey( TableModel tm, boolean forceRowid ) throws SQLException {
        try {
        	if (!forceRowid) {
                List constraints = getConstraints(tm.getCatalog(), tm.getSchema(), tm.getName(), ConstraintModel.PRIMARY_KEY);
	            if (constraints.isEmpty() && !tm.getType().equals("VIEW")) {
	            	setPseudoKeys(tm);
	            	if (tm.getPrimaryKey()==null) {
	            		Logger.println("TABLE "+ tm.getName() + " Primary Keys are defined as pseudo",Logger.DEBUG);
	            	}
	            } else if (!constraints.isEmpty()){
                    tm.setPrimaryKey((ConstraintModel) constraints.get(0));   
                }
        	}
            if (forceRowid || (tm.getPrimaryKey()==null && !tm.getType().equals("VIEW"))) {
            	SQLRecord record = new SQLRecord(9);
                record.setColumn(0, "TYPE"      ,String.class,"COLUMN");
                record.setColumn(1, "ORDINAL_POSITION" ,Integer.class,new Integer(1));
                record.setColumn(2, "COLUMN_NAME",String.class, "ROWID");
                record.setColumn(3, "TABLE_NAME" ,String.class, tm.getName());
                record.setColumn(4, "DATA_TYPE"  ,Integer.class, new Integer(Types.OTHER));
                record.setColumn(5, "TYPE_NAME"  ,String.class, "ROWID");
                record.setColumn(6, "TABLE_CAT"  ,String.class, tm.getCatalog());
                
                ColumnModel cm=new ColumnModel(record);
            	tm.getColumns().add(cm);
            	record = new SQLRecord(2);
                record.setColumn(0, "NAME"      ,String.class,"ROWID");
                record.setColumn(1, "TABLE_NAME" ,String.class,tm.getName());
                ConstraintModel model = new ConstraintModel(ConstraintModel.PRIMARY_KEY, record);
                SQLRecord rec = (SQLRecord) cm.getRecord().clone();
                rec.setValue("ORDINAL_POSITION", new Integer(1));
                model.addColumn(new ColumnModel(rec));
                tm.setPrimaryKey(model);
            	Logger.println("TABLE "+ tm.getName() + " has NO primary key, ROWID used",Logger.DEBUG);
            	
            }
             
        } catch (SQLException se) {
            if (se.getErrorCode()==0) {
                Logger.println("JDBC Driver does not support primary keys",Logger.WARNING);
            } else {
                throw se;
            }
        }
        
    }
    
    /**
     * set a pseudo primary key based on a unique index found for the table
     * supplied as parameter. 
     * @param tm TableModel, the table to find the pseudo key for
     * @throws SQLException
     */
    private void setPseudoKeys(TableModel tm) throws SQLException {
    	List indexes=getIndices(tm.getCatalog(),tm.getSchema(),tm.getName(),true);
    	if (indexes!=null) {
    		Iterator it=indexes.iterator();
    		IndexModel best=null;
            while (it.hasNext()) {
            	IndexModel idx= (IndexModel) it.next(); 
            	if (idx.isUnique() && (best==null  || best.getColumns().size()>idx.getColumns().size())) {
            		best=idx;
            	}
            }
            if (best==null) return;
            SQLRecord record = new SQLRecord(2);
            record.setColumn(0, "NAME"      ,String.class,"ROWID");
            record.setColumn(1, "TABLE_NAME" ,String.class,tm.getName());
            ConstraintModel model = new ConstraintModel(ConstraintModel.PRIMARY_KEY, record);
            for (int i=0;i<tm.getColumns().size();i++) {
                ColumnModel cm=(ColumnModel) tm.getColumns().get(i);
                for (int j=0;j<best.getColumns().size();j++) {
                	if (((ColumnModel) best.getColumns().get(j)).getName().equals(cm.getName())) {
                    	model.addColumn(cm);
                	}
                }
                
            }
            tm.setPrimaryKey(model);
    	}
    }
    
    
    /**
     * set the Indexes for this table
     * @param proc ProcedureModel, the model to set the parameters to
     * @throws SQLException
     * @see {@link #getColumns(String, String, String, String)}
     */    
    public void setIndices(TableModel table, boolean onlyUnique) throws SQLException{
        List indices = getIndices(table.getCatalog(), table.getSchema(), table.getName(), onlyUnique);
        table.setIndices(indices);
    }
    
    /**
     * get a list of Indices for the given parameters.
     * @param catalog String, a possible catalog name, null if not used
     * @param schemaPattern String, a possible schemaPattern, null if not used
     * @param tablePattern, a possible tablePattern matching the name of the TableModel
     * @param onlyUnique, true if only unique indexes should be retrieved
     * @return List of IndexModels, returns a List even if no indices where found
     * @throws SQLException
     */
    public List getIndices(String catalog, String schemaPattern, String tablePattern, boolean onlyUnique)  throws SQLException{
    	HashMap indices=new HashMap();
        ResultSet rs= null;
        
        if (isOracleDriver()) {
            rs = getOracleIndexes(schemaPattern, tablePattern,onlyUnique);
        } else {
            rs=getMetaData().getIndexInfo(catalog,schemaPattern,tablePattern,onlyUnique,true);
        }
    	while (rs.next()) {
            SQLRecord record = SQLRecord.createRecord(rs);
            IndexModel index =  new IndexModel(createSubSet(record, new String[] {"TABLE_CAT","TABLE_SCHEM","TABLE_NAME","NON_UNIQUE","INDEX_QUALIFIER","INDEX_NAME"})); 
            ColumnModel column = new ColumnModel(createSubSet(record, new String[] {"TABLE_CAT","TABLE_SCHEM","TABLE_NAME","ORDINAL_POSITION","COLUMN_NAME","ASC_OR_DESC"}));
            
            String name=index.getName();
    		if (name!=null) {
	    		IndexModel idx=(IndexModel) indices.get(name);
	    		if (idx==null) {
	    			idx=new IndexModel(record);
	    			indices.put(name,idx);
	    		}
	    		idx.setColumn(column.getSequence(),column);
    		}
    		
    	}
    	rs.close();
        if (isOracleDriver() && m_prep!=null) {
            m_prep.close();
        }
    	return new ArrayList(indices.values());
    }
	
    /**
     * get a List of Procedures defined in this database without possible parameters
     * for the given catalog,schema and pattern.
      * <p>Setting any of the parameters to null will ignore them.
     * Setting either catalog or schemaPattern is usually necessary.
     * @param catalog String, a possible catalog name, null if not used
     * @param schema  String, a possible schemaPattern, null if not used
     * @param pattern String, a possible tablePattern, null if not used.
     * @return List of ProcedureModel, returns a List even if no procedures where found
     * @throws SQLException
     */
    public List getProcedures(String catalog,String schema, String pattern) throws SQLException {
        return getProcedures(catalog, schema, pattern, true);
    }
    
    /**
     * get a List of Procedures defined in this database for the
     * given catalog,schema and pattern.
      * <p>Setting any of the parameters to null will ignore them.
     * Setting either catalog or schemaPattern is usually necessary.
     * @param catalog String, a possible catalog name, null if not used
     * @param schema  String, a possible schemaPattern, null if not used
     * @param pattern String, a possible tablePattern, null if not used.
     * @param includeParms boolean, true if you want the parameters for this
     * procedure loaded as well, false otherwise. 
     * <p><i>Note:</i> Setting this to true might slow down the result</p> 
     * @return List of ProcedureModel, returns a List even if no procedures where found
     * @throws SQLException
     */
	public List getProcedures(String catalog,String schema, String pattern,boolean includeParms) throws SQLException {
		ArrayList result=new ArrayList();
		ResultSet rs=getMetaData().getProcedures(catalog,schema,pattern);
		while (rs.next()) {
			ProcedureModel proc=new ProcedureModel(SQLRecord.createRecord(rs));
            if (includeParms) setParameters(proc);
			result.add(proc);
		}
		rs.close();
		return result;
	}
    
    /**
     * set the parameters for this procedure
     * @param proc ProcedureModel, the model to set the parameters to
     * @throws SQLException
     * @see {@link #getParameters(String, String, String)}
     */    
    public void setParameters(ProcedureModel proc) throws SQLException{
        List param = getParameters(proc.getCatalog(), proc.getSchema(), proc.getName());
        if (proc.getType()==ProcedureModel.FUNCTION) {
            proc.setReturnValue((ParameterModel) param.get(0));
            param.remove(0);
        }	
        proc.setParameters(param);
    }
    
    /**
     * get the parameters belonging to the procedure 
     * @param catalog String, catalog or null
     * @param schema String, schema or null
     * @param procName String, the name of the procedure, leaving
     * this blank will return the parameters of all procedures
     * @return List of {@link ParameterModel}
     * @throws SQLException
     */
    public List getParameters(String catalog, String schema, String procName) throws SQLException {
        ArrayList result=new ArrayList();
		ResultSet rs=getMetaData().getProcedureColumns(
			catalog==null?"":catalog,
			schema,
			procName,
			null);
		while (rs.next()) {
			ParameterModel param=new ParameterModel(SQLRecord.createRecord(rs));
			result.add(param);
			
		}
		rs.close();
		return result;
			
    }
    
	/**
	 * get the connection to the Database
	 * @return Connection, the connection to the Database
	 */
	public Connection getConnection() {
		return m_con;
	}

	/**
	 * set the connection to the Database to
     * use for the MetaData
	 * @param con Connection, the connection to the Database
	 */
	public void setConnection(Connection con) {
		m_con = con;
		m_dbMeta=null;
	}
    
    /**
     * get a List of Constraints (Primary | Foreign ) defined in this database for the
     * given catalog,schema and pattern.
      * <p>Setting any of the parameters to null will ignore them.
     * Setting either catalog or schemaPattern is usually necessary.
     * @param catalog String, a possible catalog name, null if not used
     * @param schema  String, a possible schemaPattern, null if not used
     * @param pattern String, a possible tablePattern, null if not used.
     * @param type String, the type of constraint to get <code>( ConstraintModel.PRIMARY_KEY | Constraint.FOREIGN_KEY )</code>
     * or null for both types
     * @return List of ProcedureModel, returns a List even if no procedures where found
     * @throws SQLException
     */
    public List getConstraints(String catalog, String schema, String table, String type) throws SQLException {
        String lastName=null;
        ArrayList list= new ArrayList();
        ConstraintModel model= null;
        SQLRecord record= null;
        
        if (type==null || type==ConstraintModel.PRIMARY_KEY) {
            ResultSet rs= getMetaData().getPrimaryKeys(catalog, schema, table);
            while (rs.next()) {
                record = SQLRecord.createRecord(rs);
                String name = rs.getString("PK_NAME");
                if ((name==null && lastName==null) || !name.equals(lastName)) {
                    model= new ConstraintModel(ConstraintModel.PRIMARY_KEY,createSubSet(record, new String[] {"TABLE_CAT","TABLE_SCHEM", "TABLE_NAME", "PK_NAME"},"PK"));
                    list.add(model);
                } 
                    
                ColumnModel column=new ColumnModel(createSubSet(record, new String[] {"TABLE_CAT","TABLE_SCHEM", "TABLE_NAME", "COLUMN_NAME","KEY_SEQ"}));
                renameColumn(column.getRecord(),"KEY_SEQ","ORDINAL_POSITION");
                model.addColumn(column);
                lastName=name;
            }
        }
        
        model= null;
        record= null;
        lastName=null;
        if (type==null || type==ConstraintModel.FOREIGN_KEY) {
            ResultSet rs= getMetaData().getImportedKeys(catalog, schema, table);
            while (rs.next()) {
                record = SQLRecord.createRecord(rs);
                String name = rs.getString("FK_NAME");
                if ((name==null && lastName==null) || !name.equals(lastName)) {
                    model= new ConstraintModel(ConstraintModel.FOREIGN_KEY,createSubSet(record, new String[] {"FKTABLE_CAT","FKTABLE_SCHEM","FKTABLE_NAME","FK_NAME","UPDATE_RULE","DELETE_RULE","DEFERRABILITY"},"FK"));
                    list.add(model);
                }
                ColumnModel column=new ColumnModel(createSubSet(record, new String[] {"FKTABLE_CAT","FKTABLE_SCHEM","FKTABLE_NAME","FKCOLUMN_NAME","KEY_SEQ"},"FK"));
                renameColumn(column.getRecord(),"KEY_SEQ","ORDINAL_POSITION");
                model.addColumn(column);
                column=new ColumnModel(createSubSet(record, new String[] {"PKTABLE_CAT","PKTABLE_SCHEM","PKTABLE_NAME","PKCOLUMN_NAME","KEY_SEQ"},"PK"));
                renameColumn(column.getRecord(),"KEY_SEQ","ORDINAL_POSITION");
                model.addColumn(column);
                lastName=name;
            }
        }        
        return list;
    }

    private SQLRecord createSubSet(SQLRecord record, String[] columns) throws SQLException {
        return createSubSet(record, columns,null);    
    }
    
    private SQLRecord createSubSet(SQLRecord record, String[] columns, String prefix) throws SQLException {
        SQLRecord sub =new SQLRecord(columns.length);
        for (int i=0;i<columns.length;i++) {
            int idx= record.getIndex(columns[i]);
            if (prefix!=null && columns[i].startsWith(prefix)) {
                int start=prefix.length();
                if (columns[i].charAt(start)=='_') start++;
                columns[i] =columns[i].substring(start);
            }
            if (idx!=-1) { 
                sub.setColumn(i, columns[i], record.getColumnType(idx), record.getValue(idx));
            }
        }
        return sub;
    }
    
    /** rename a column in an SQLRecord
     * @param record SQLRecord
     * @param oldName String, the old name of the column
     * @param newName String, the new name of the column
     */
    private void renameColumn(SQLRecord record, String oldName, String newName) {
        int idx=record.getIndex(oldName);
        if (idx!=-1) {
            record.setColumn(idx, newName);
        }      
    }
    
    /**
     * check if the driver used is Oracle
     * <p>Used for {@link #getIndices(String, String, String, boolean)}
     * to fix speed problem on updating statistics</p>
     * @return
     * @throws SQLException
     */
    private boolean isOracleDriver() throws SQLException {
        return "Oracle".equals(getMetaData().getDatabaseProductName());
    }
    
    /**
     * Alternative method to get Indexes from an Oracle database
     * <p>Used for {@link #getIndices(String, String, String, boolean)}
     * to fix speed problem on updating statistics which the Oracle driver
     * feels the need to do</p>
     * @param schemaPattern String, a possible schemaPattern, null if not used
     * @param tablePattern, a possible tablePattern matching the name of the TableModel
     * @param onlyUnique, true if only unique indexes should be retrieved
     * @return ResultSet, a ResultSet equal in content to what {@link DatabaseMetaData#getIndexInfo(String, String, String, boolean, boolean)}
     * would return
     * @throws SQLException
     */
    private ResultSet getOracleIndexes(String schemaPattern, String tablePattern, boolean onlyUnique) throws SQLException {
        //Fix for Bug in Oracle driver #4999817
        m_prep = getMetaData().getConnection().prepareStatement(
            "SELECT NULL TABLE_CAT, I.TABLE_OWNER TABLE_SCHEM,I.TABLE_NAME, I.INDEX_NAME, INDEX_TYPE INDEX_QUALIFIER,COLUMN_NAME," +
            " DECODE(UNIQUENESS,'UNIQUE',0,1) NON_UNIQUE, COLUMN_POSITION ORDINAL_POSITION, DESCEND ASC_OR_DESC FROM ALL_INDEXES I, ALL_IND_COLUMNS C " +
            " WHERE I.TABLE_OWNER LIKE ? AND I.TABLE_NAME LIKE ? AND I.INDEX_NAME = C.INDEX_NAME AND I.OWNER = C.INDEX_OWNER AND " +
            " ((UNIQUENESS='UNIQUE' AND 1=?) OR (0=?))" +
            " ORDER BY TABLE_CAT, TABLE_SCHEM, I.TABLE_NAME,ORDINAL_POSITION"
        );
        m_prep.setString(1, schemaPattern==null?"%":schemaPattern );
        m_prep.setString(2, tablePattern  );
        m_prep.setInt(   3, onlyUnique?1:0);
        m_prep.setInt(   4, onlyUnique?1:0);
        ResultSet rs= m_prep.executeQuery();
        return rs;
        
    }

    /**
     * get a Types2Java object for transforming types between
     * the database and Java
     * @return Types2Java, null if the {@link Connection} or {@link DatabaseMetaData}
     * cannot be found
     * @since 1.00
     */
    public TypesInfo getTypeInfo() {
        if (m_sqlTypes==null) {
            try {
            	m_sqlTypes = new TypesInfo(m_con);
                getMetaData();
            } catch (SQLException sqex) {
                // Don't do anything just return null
            }
        }
        return m_sqlTypes;
    }
    
}
