/* 	SelectDescriptor.java
 *	Copyright (c) 2006, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Oct 14, 2006
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * @#History 
 * [When] 		[Who] 				[What] 
 * Oct 21, 2006 	dennis@brains2b.nl	Don't split outer joins
 * TODO Should not return KEYWORDS themselves. Needs fix in DataC first
 */
package org.brains2b.sql.meta;

/**
 * Describe a select statement in terms of the different keywords which
 * makes it up
 * 
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.10.1 [21-10-2006]
 */
public class SelectDescriptor {

	private String[] m_sqlKeys = new String[7];
	
    /**
     * Create an SelectDescriptor on the basis of an select-statement
     * @param sql String, a valid select statement
     */
	public SelectDescriptor(String sql) {
		super();
		parse(sql);
	}
	
	/**
     * get the complete statement equals to the 
     * select statement which was passed in
     * @see Object#toString() 
	 */
	public String toString() {
		return m_sqlKeys[0];
	}
	
    /**
     * get a comma-seperated list of columns for this
     * select statement
     * @return String, a comma-seperated list of columms. 
     * Could contain only <code>*</code> dependant on the 
     * select statement
     */
	public String getColumns() {
		return m_sqlKeys[1];
	}
	
    /**
     * get a comma-seperated list of tables from the FROM-part
     * of this select statement
     * @return String, a comma-seperated list of tables. 
     */
	public String getFrom() {
		return m_sqlKeys[2];
	}
	
    /**
     * get the WHERE clause of this select statement
     * @return String, the WHERE clause of this select statement
     * without the keyword. Can be null if no WHERE-statement was defined
     */
	public String getWhere() {
		return m_sqlKeys[3];
	}
	
    /**
     * get a comma-seperated list of columns from the GROUP BY-part
     * of this select statement
     * @return String, a comma-seperated list of columms. 
     * Can be null if no GROUP BY-statement was defined
     */
	public String getGroupBy() {
		return m_sqlKeys[4];
	}
	
    /**
     * get the HAVING clause of this select statement
     * @return String, the HAVING clause of this select statement
     * without the keyword. Can be null if no HAVING-statement was defined
     */
	public String getHaving() {
		return m_sqlKeys[5];
	}
	
    /**
     * get a comma-seperated list of columns from the ORDER BY-part
     * of this select statement
     * @return String, a comma-seperated list of columms. 
     * Can be null if no ORDER BY-statement was defined
     */
	public String getOrderBy() {
		return m_sqlKeys[6];
	}
	
    /**
     * parse the Select statement into its different parts
     * Reads the SQL statement from end to start 
     * @param sql String
     */
	private void parse(String sql) {
		if (sql==null) return;
		String usql=sql.toUpperCase();
		int pos=usql.length();
		pos =parsePart(usql,sql,"ORDER BY",6,pos);
		pos =parsePart(usql,sql,"HAVING",5,pos);
		pos =parsePart(usql,sql,"GROUP BY",4, pos);
		pos =parsePart(usql,sql,"WHERE",3,pos);
		pos =parsePart(usql,sql,"FROM",2,pos);
		pos =parsePart(usql,sql,"SELECT",1,pos);
		m_sqlKeys[0]=sql;
	}
	
    /**
     * String parse part the SQL string on the basis of the KEYWORD
     * @param usql String, the SQL statement in uppercase
     * @param sql String, the SQL Statement
     * @param key String, the keyword to scan for
     * @param idx int, index for storage (same position on basis of keyword)
     * @param pos int, the end position to scan for the keyword
     * @return
     */
	private int parsePart(String usql, String sql, String key, int idx, int pos) {
		int c=-1;
		c= usql.indexOf(key);
		if (c>=0 && c<pos) {
			m_sqlKeys[idx]=sql.substring(c,pos);
			pos=c;
		}
		return pos;
	}
	
}
