/* 	TableModel.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 26-mrt-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Oct 23, 2007	dennis@brains2b.nl		Implement equals()		
*/
package org.brains2b.sql.meta;

import java.util.List;

import org.brains2b.sql.SQLRecord;

/** Wrapper around table definition from DatabaseMetaData
 * <p>Class is read-only, use SquelerDDL:TableDDL for changing columns</p>
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 1.01 [Oct 23, 2007]
 * @see SQLMetaData
 * @see SQLRecord
 */
public class TableModel implements SQLMetaModel {

    private SQLRecord m_record;
    private List m_columns;
    private List m_indices;
    private ConstraintModel m_primaryKey;
    
    /**
     * Constructor 
     * only used by TableDDL
     *
     */
    protected TableModel() {
        super();
    }
    
     /**
     * Create a TableModel on the basis of this SQLRecord
     * @param record SQLRecord, the record containing the 
     * information for this table
     */
    public TableModel(SQLRecord record) {
        m_record =record;
    }
     
    /**
     * {@inheritDoc}
     */
    public String getName() {
        return m_record.getString("TABLE_NAME");
    }
    
    /**
     * get the Catalog for this TableModel
     * @return String, null if this table does not
     * have a catalog 
     */
    public String getCatalog() {
        return m_record.getString("TABLE_CAT");
    }
    
    /**
     * get the Schema for this TableModel
     * @return String, null if this table does not
     * have a schema 
     */
    public String getSchema() {
        return m_record.getString("TABLE_SCHEM");
    }
    
    /**
     * get optional remarks for this table
     * @return String, the remarks for this table
     */
    public String getRemarks() {
        return m_record.getString("REMARKS");
    }
    
    /**
     * {@inheritDoc}
     */
    public SQLRecord getRecord() {
        return m_record;
    }

    /**
     * {@inheritDoc}
     */
    public String getType() {
        return (String) m_record.getString("TABLE_TYPE");
    }
    
    /**
     * get the columns for this Table
     * @return List of ColumnModel
     */
    public List getColumns() {
        return m_columns;
    }
    
    /**
     * set the columns for this TableModel
     * @param List of ColumnModel
     */
    protected void setColumns(List columns) {
        m_columns = columns;
    }

    /**
     * get the indices for this Table
     * @return List of IndexModel
     */
    public List getIndices() {
        return m_indices;
    }

    /**
     * set the indices for this Model
     * @param List of IndexModel
     */
    protected void setIndices(List indices) {
        m_indices = indices;
    }
    
    /** 
     * set the primary key for this table
     * @param ConstraintModel  
     * @throws IllegalArgumentException if the ConstraintModel is not
     * of type {@link ConstraintModel#PRIMARY_KEY}
     */
    protected void setPrimaryKey(ConstraintModel primaryKey) {
        if (primaryKey!=null && primaryKey.getType()!=ConstraintModel.PRIMARY_KEY) {
            throw new IllegalArgumentException("You can only set a primary key to the table");
        }
        m_primaryKey = primaryKey;
    }

    /** set the record for this Model
     *  @param record SQLRecord
     */
    protected void setRecord(SQLRecord record) {
        m_record = record;
    }

    /**
     * get the primary key for this table in the form
     * of a {@link ConstraintModel} of type {@link ConstraintModel#PRIMARY_KEY}
     * @return {@link ConstraintModel}
     */
    public ConstraintModel getPrimaryKey() {
        return m_primaryKey;
    }

    public boolean equals(Object obj) {
    	if (!(obj instanceof TableModel)) return false;
    	TableModel tm = (TableModel) obj;
    	boolean b = m_record.equals(tm.getRecord());
    	if (!b || m_columns==null || m_columns.isEmpty()) return b;
    	List cols = tm.getColumns();
    	if (cols==null || cols.size()!=m_columns.size()) return false;
    	for (int i=0; i<m_columns.size();i++) {
    		b = m_columns.get(i).equals(cols.get(i));
    		if (!b) return b;
    	}
    	b= (m_primaryKey==null)?b:m_primaryKey.equals(tm.getPrimaryKey());
    	if (!b || m_indices==null || m_indices.isEmpty()) return b;
    	List indices = tm.getColumns();
    	if (indices==null || indices.size()!=m_indices.size()) return false;
    	for (int i=0; i<m_indices.size();i++) {
    		b = m_indices.get(i).equals(indices.get(i));
    		if (!b) return b;
    	}
    	return true;
    }
}
