/* 	TypesInfo.java
 * 
 * 	Created on: 20 mei 2001, 23:28
 * 	Created by: Dennis Groenendijk
 * 
 * Copyright (c) 2001, Dennis Groenendijk
 * Copyright (c) 2002-2008, Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 /*
 * History
 * Ratl 			   25-05-2001 Added support for Date, Time, Timestamp types
 * Ratl 			   06-06-2001 Added support for INTEGER and SMALLINT
 * dennis@brains2bl.nl 19-12-2002 Added support for CHAR
 * dennis@brains2bl.nl 18-02-2003 Added getOracleDatatype() used by squelerDDL
 * 													hidden constructor, added comments
 * dennis@brains2b.nl  28-07-2003 getJavaType() ignored char fields.
 * dennis@brains2b.nl  28-04-2004 return String if datatype is LONGVARBINARY (LONG RAW)
 * dennis@brains2b.nl  12-02-2005 Return long for numbers where column length > 10
 * dennis@brains2b.nl	  02-08-2006 Option for setting the size of the Java types from Oracle types
 * dennis			  09-08-2006 Fix voor making too many columns into a float
 * dennis@brains2b.nl	  30-08-2006 Added method to check if dataTypes were set
 * dennis@brains2b.nl	  07-09-2006 Fix for large type
 * 17-01-2008	dennis@brains2b.nl      changed to an instanstiable class
 * 29-01-2008	dennis@brains2b.nl		Detect lob datatypes correctly
 * 18-03-2008	dennis@brains2b.nl		Detect stream datattypes correctly
 * 13-04-2008	dennis@brains2b.nl		Possibility of DEFAULT TypesInfo, fix for LONGVARCHAR
 * TODO CHAR should be implemented as char if columnlength==1
 * TODO if type cannot be found and Exception should be thrown, not a message returned
 */
package org.brains2b.sql.meta;

import java.sql.Blob;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.Collection;
import java.util.HashMap;

import org.brains2b.log.Logger;
import org.brains2b.sql.meta.types.Stream;

/**
 * Types to Java 
 * Static class to translate java.sql.Types into appropriate Java types
 * At the moment only primitives, String and java.sql.Date,Time,Timestamp are used
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.32.1 [13-04-2008]
 */
public class TypesInfo extends java.lang.Object {
    
	public static final TypesInfo DEFAULT = new TypesInfo();
	
    public static final int SMALLEST=22;
    public static final int OPTIMIZE=10;
    public static final int LARGE=9;
    
    private int m_dataModel=OPTIMIZE;
    private HashMap m_types;
    private String m_dbUrl;
    
    TypesInfo() {
    	super();
    }
    
    public TypesInfo(Connection con) {
    	super();
    	try {
        	setTargetTypes(con);
        	m_dbUrl = con.getMetaData().getURL();
    	} catch (SQLException sqex) {
    		Logger.println(sqex.getMessage(),Logger.DEBUG);
    	}
    }
   
    private void addTargetTypes(int dataType, String typeName) {
		if (m_types==null) m_types=new HashMap();
		if (!m_types.containsKey(new Integer(dataType)))
			m_types.put(new Integer(dataType),typeName);
    }
    
    /**
   	 * get the <code>java.sql.Types</code> value for a given class
   	 * @param c Class, the class for which the java.sql.Types value is requested
   	 * @return int, a number representing a constant from <code>java.sql.Types</code>
   	 */
    public int getDataType(Class c) {
    	if (c==Boolean.TYPE)
			return Types.BIT;
		else if (c==Short.TYPE)
			return Types.SMALLINT;
		else if (c==Integer.TYPE)
			return Types.INTEGER;
		else if (c==Long.TYPE||c==Float.TYPE||c==Double.TYPE)
			return Types.DECIMAL;
		else if (c==String.class)
			return Types.VARCHAR;
		else if (c==java.util.Date.class)
			return Types.DATE;
		else if (c==Timestamp.class)
			return Types.TIMESTAMP;
		else
			return Types.OTHER;
    }
    
    /**
	 * gets OracleDatatype as String from a type as defined in java.sql.Types
	 * @param dataType, int the datatype that needs to be one of the types defined in 
	 * java.sql.Types
	 * @return String, a type name that is recognized on an Oracle database
	 */
    private String getDefaultDatatype(int dataType) {
    	String type= null;
        switch (dataType) {
            case java.sql.Types.BIT: 		/* value= -7 */
            case java.sql.Types.TINYINT: 	/* value= -6 */
            case java.sql.Types.INTEGER:
            case java.sql.Types.SMALLINT:
            case java.sql.Types.DECIMAL:
            case java.sql.Types.NUMERIC:
				type="NUMBER";
                break;
            case java.sql.Types.CHAR:
            case java.sql.Types.VARCHAR:
                type="VARCHAR2";
                break;
            case java.sql.Types.DATE:
            case java.sql.Types.TIME:
            case java.sql.Types.TIMESTAMP:
                type="DATE";
                break;
            case java.sql.Types.OTHER:
            	type="BLOB";
            	break;
            default:
                type="/* value: " +dataType + " cannot be determinend */";
        }
        return type; 
    }
	/**
	 * gets a String representation of a valid java type on the basis of JDBC column information
	 * <p>Normally the dataType is enough to make a good determination, except in the case of Numbers.
	 * <UL>For numbers the following rules appy
	 * <LI>If decimaldigits is 0 and columnsize &lt 10 the number is made into an int
	 * <LI>If decimaldigits is 0 and columnsize &gt 10 the number is made into a long
	 * <LI>If decimaldigits &gt 0 and columnsize &lt 10 the number is made into a float
	 * <LI>If decimaldigits &gt 0 and columnsize &gt 10 the number is made into a double
	 * <p>Types.OTHER is returned as a generic Object.
	 * <p><i>Note:</i>Please be sure to understand that not all Types are defined and translated here
	 * if you miss one add it yourself or mail me.
	 * @param dataType int, A valid datatype from java.sql.Types
	 * @param columnsize int, The length of the column
	 * @param decimaldigits int, The number of decimal digits
	 * @return String, the String representation of a valid Java type.
	 */
    public String getJavaTypeName(int dataType,int columnsize,int decimaldigits) {
        String type="";
        switch (dataType) {
            case java.sql.Types.BIT: 		/* value= -7 */
                type="boolean"; 
                break;
            case java.sql.Types.TINYINT: 	/* value= -6 */
                type="short";
                break;
            case java.sql.Types.INTEGER:
                type="int";
                break;
            case java.sql.Types.SMALLINT:
                type="short";
                break;
            case java.sql.Types.DECIMAL:
            	case java.sql.Types.DOUBLE:
			case java.sql.Types.NUMERIC:
                if (decimaldigits==0) {
                    if (columnsize<m_dataModel)
						type="int";
					else
						type="long";
                } else if (columnsize<SMALLEST && decimaldigits<m_dataModel)
					type="double";
				else
				    type="float";
                break;
            case java.sql.Types.CHAR:
                type="String";
                break;
            case java.sql.Types.VARCHAR:
                type="String";
                break;
            case java.sql.Types.DATE:
                type="Date";
                break;
            case java.sql.Types.TIME:
            case java.sql.Types.TIMESTAMP:
                type="Timestamp";
                break;
            case java.sql.Types.OTHER:
            	type="Object";
            	break;
            case java.sql.Types.LONGVARBINARY:
            case java.sql.Types.LONGVARCHAR:
            	type="Stream";
            	break;
            case java.sql.Types.CLOB:
				type="Clob";
				break;
            case java.sql.Types.BLOB:
            	type="Blob";
				break;
            default:
                type="/* value: " +dataType + " cannot be determinend */";
                
        }
        return type;
    }
    	
	/**
	 * get the JavaType for a specific dataType as defined in java.sql.Types
	 * <p>This method returns only one of two values PRIMITIVE or CLASS
	 * @param dataType int, the datatype as defined in java.sql.Types
	 * @return int, the java type for the dataType submitted, either returns
	 * one of the constants PRIMITIVE or CLASS
	 * <p><i>Note:</i>Please be sure to understand that the determination is not done for all Types 
	 * if you miss one add it yourself or mail me.
	 */
    public boolean isPrimitive(int dataType) {
        return getJavaType(dataType, 0, 0).isPrimitive();
    }
    
	/**
	 * gets a Class as a valid java type on the basis of JDBC column information
	 * <p>Normally the dataType is enough to make a good determination, except in the case of Numbers.
	 * <UL>For numbers the following rules appy
	 * <LI>If decimaldigits is 0 and columnsize &lt 22 the number is made into an int
	 * <LI>If decimaldigits is 0 and columnsize &gt 22 the number is made into a long
	 * <LI>If decimaldigits &gt 0 and columnsize &lt 22 the number is made into a float
	 * <LI>If decimaldigits &gt 0 and columnsize &gt 22 the number is made into a double
	 * <p>Types.OTHER is returned as a generic Object.
	 * <p><i>Note:</i>Please be sure to understand that not all Types are defined and translated here
	 * if you miss one add it yourself or mail me.
	 * @param dataType int, A valid datatype from java.sql.Types
	 * @param columnsize int, The length of the column
	 * @param decimaldigits int, The number of decimal digits
	 * @return Class, the class representing a valid data-type for Java. Will return the "WrapperClass".TYPE
	 * for primitives
	 */
    public Class getJavaType(int dataType,int columnsize,int decimaldigits) {
        Class type=null;
        switch (dataType) {
            case java.sql.Types.BIT: 		/* value= -7 */
                type = Boolean.TYPE;
                break;
            case java.sql.Types.INTEGER:
                type = Integer.TYPE;
                break;
            case java.sql.Types.TINYINT: 	/* value= -6 */
            case java.sql.Types.SMALLINT:
                type = Short.TYPE;
                break;
            case java.sql.Types.DECIMAL:
            case java.sql.Types.DOUBLE:
            case java.sql.Types.NUMERIC:	/* value= 2 */
                if (decimaldigits==0) {
                    if (columnsize<m_dataModel)
						type = Integer.TYPE;
					else
						type = Long.TYPE;
                } else if (columnsize<SMALLEST && decimaldigits<m_dataModel)
					type = Double.TYPE;
				else
				    type = Float.TYPE;
                break;
            case java.sql.Types.CHAR:
            case java.sql.Types.VARCHAR:
                type = String.class;
                break;
            case java.sql.Types.DATE:
                type = java.util.Date.class;
                break;
            case java.sql.Types.TIME:
            case java.sql.Types.TIMESTAMP:
                type = Timestamp.class;
                break;
            case java.sql.Types.OTHER:
            	type = Object.class;
            	break;
            case Types.LONGVARCHAR:
            	type = Stream.class;
            	break;
            case Types.LONGVARBINARY:
            	type = Stream.class;
            	break;
            case Types.CLOB:
            	type = Clob.class;
            	break;
            case Types.BLOB:
            	type = Blob.class;
            default:
            	type = String.class;
        }
        return type;
    }
    
   	public String getLimitRows(int count) {
		if (m_dbUrl.indexOf("oracle")!=-1)
			return " WHERE ROWNUM < " + count; 
		return " LIMIT " + count;
    }
    
	public String getSQLType(int dataType) {
		String s=null;
		if (isTargetTypesSet())
			s= (String) m_types.get(new Integer(dataType)); 
		if (s==null) s=getDefaultDatatype(dataType);
		return s;
    }
    
    public Collection getTypeNames() {
        if (isTargetTypesSet()) return null;
        return m_types.values();
    }
    
    public HashMap getTypes() {
        return m_types;
    }
    
    private final boolean isTargetTypesSet() {
		return m_types!=null;
    }
    
    public void setDataModel(int dataModel) {
    	m_dataModel=dataModel;
    }
    
     /**
     * set the target types from the DatabaseMetaData information
     * @throws SQLException if {@link DatabaseMetaData#getTypeInfo()} fails
     */
    private final void setTargetTypes(Connection con) throws SQLException {
		final DatabaseMetaData meta = con.getMetaData();
    	final ResultSet rs= meta.getTypeInfo();
        while (rs.next()) {
            // TODO: Check impact of removing this
            if (meta.getDriverName().indexOf("Oracle")!=-1 && rs.getInt("DATA_TYPE") == Types.TIMESTAMP) {
                addTargetTypes(rs.getInt("DATA_TYPE"),"DATE");                    
            } else {
			    addTargetTypes(rs.getInt("DATA_TYPE"),rs.getString("TYPE_NAME"));
            }
		}
		rs.close();
    }
}
